/* === S Y N F I G ========================================================= */
/*!	\file string_decl.h
**	\brief Template Header
**
**	$Id$
**
**	\legal
**	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
**
**	This package is free software; you can redistribute it and/or
**	modify it under the terms of the GNU General Public License as
**	published by the Free Software Foundation; either version 2 of
**	the License, or (at your option) any later version.
**
**	This package is distributed in the hope that it will be useful,
**	but WITHOUT ANY WARRANTY; without even the implied warranty of
**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
**	General Public License for more details.
**	\endlegal
*/
/* ========================================================================= */

/* === S T A R T =========================================================== */

#ifndef __SYNFIG_STRING_DECL_H
#define __SYNFIG_STRING_DECL_H

/* === H E A D E R S ======================================================= */

/* === M A C R O S ========================================================= */

/* === T Y P E D E F S ===================================================== */

/* === C L A S S E S & S T R U C T S ======================================= */

namespace std
{
template<class _CharT> struct char_traits;
template<> struct char_traits<char>;
template<typename _Alloc> class allocator;
template<typename _CharT, typename _Traits,typename _Alloc>class basic_string;
typedef basic_string<char,char_traits<char>,allocator<char> >    string;

}; // END of namespace std

namespace synfig
{

/*!	\typedef String
**	\todo writeme
*/
typedef std::string String;

}; // END of namespace synfig

/* === E N D =============================================================== */

#endif
